/* Copyright (C) 1999 Chris Vine, G3XXF

This program is distributed under the General Public Licence, version 2.
For particulars of this and relevant disclaimers see the file
COPYRIGHT distributed with the source files.

*/

#ifndef FILESEND_H
#define FILESEND_H

#include "dlist.h"
#include <iostream.h>
#include <fstream.h>
#include <ncurses.h>
// we don't want to use the 'clear' macro in ncurses.h - it will interfere with iostream::clear()
#undef clear
#include <string.h>
#include <iomanip.h>
#include <strstream.h>
#include <time.h>
#include <signal.h>
#include <ctype.h>
#include "prog_defs.h"
#include "buffers.h"
#include "tnc.h"
#include "screen.h"

typedef unsigned char uchar;

extern Prog_func prog_func;

extern "C" void cq_alarm_handler(int);

class FilesendBuffer: public FileBuffer {
    ifstream filein;
    enum {not_sending, loading_buffer, loading_complete} file_flag;
    Tnc* tnc_ptr;
    SendWin* sendwin_ptr;
    ReceiveWin* receivewin_ptr;
    DList<Screen>* screenform_ptr;
    int convert_codes_flag;
public:
    int open_file(const char*, int);
    int load_buffer(void);
    void end_loading(void) {file_flag = loading_complete; reset();}
    FilesendBuffer(Buffer_mode a, int b, int c, Tnc* d, SendWin* e, ReceiveWin* f, DList<Screen>* g):
                                     FileBuffer(a, b, c), file_flag(not_sending), tnc_ptr(d),
                                     sendwin_ptr(e), receivewin_ptr(f),
                                     screenform_ptr(g), convert_codes_flag(FALSE) {}
    ~FilesendBuffer(void) {}
};

class CqsendBuffer: public FileBuffer {
    ifstream filein;
    enum {off, loading_cq_buffer, loading_cq_buffer_complete, waiting, close_down} autocq_flag;
    Tnc* tnc_ptr;
    MainScreen* mainscreen_ptr;
    SendWin* sendwin_ptr;
    ReceiveWin* receivewin_ptr;
    DList<Screen>* screenform_ptr;
    int rx_count;
public:
    int load_buffer(void);
    void end_loading(void) {autocq_flag = close_down;}
    CqsendBuffer(Tnc* a, MainScreen* b, SendWin* c, ReceiveWin* d, DList<Screen>* e):
      FileBuffer(FileBuffer::text, 0, 1), autocq_flag(off), tnc_ptr(a),
                                     mainscreen_ptr(b), sendwin_ptr(c), receivewin_ptr(d),
                                     screenform_ptr(e), rx_count(0) {}
    ~CqsendBuffer(void) {}
};

class BufferList: public DList<FileBuffer> {
public:
    enum Upload_status {keyboard, file};
private:
    Upload_status upload_status[MAXUSERS][2];
public:
    Upload_status get_upload_status(int stream, int port) const 
                                          {if (stream >= MAXUSERS || port > 1) return keyboard;
					   else return upload_status[stream][port];}
    void add(FileBuffer* file_b_p) {upload_status[file_b_p->get_stream()][file_b_p->get_port()] = file;
                                    DList<FileBuffer>::add(file_b_p);}
    const FileBuffer* extract(void) {const FileBuffer* file_b_p = DList<FileBuffer>::extract();
                                     upload_status[file_b_p->get_stream()][file_b_p->get_port()] = keyboard;
				     return file_b_p;}
    BufferList(void);
};

#endif
